//General LED management
#include <FastLED.h>
#define NUM_LEDS 145
#define DATA_PIN 6
#define BRIGHTNESS  50
CRGB leds[NUM_LEDS];

//for the nonblocking code
#define framerate 33 //how many milliseconds to wait between updating the display
unsigned long previousdisplaymillis = 0; //This stores the time when the LED display was last updated
#define sprite1speed 100 // how many milliseconds to wait between updating sprite 1 position
unsigned long previoussprite1millis = 0; //This stores the time when the sprite was last moved
#define backgroundframerate 16 // how many ms to wait between updating the background
unsigned long previousbackgroundmillis = 0; //When was the background last updated

//For the sprite moving stuff
int LEDposx = 12; //starting x position for the LED
int LEDposy = 4; //starting y posiiton for the LED
const byte LEDarray[23][9] =  {{145, 145, 145, 145, 0, 145, 145, 145, 145} , {145, 145, 1, 2, 3, 4, 5, 145, 145} , {145, 12, 11, 10, 9, 8, 7, 6, 145},  {145, 13, 14, 15, 16, 17, 18, 19, 145} , {145, 26, 25, 24, 23, 22, 21, 20, 145} , {145, 27, 28, 29, 30, 31, 32, 33, 145} , {145, 40, 39, 38, 37, 36, 35, 34, 145},  {145, 41, 42, 43, 44, 45, 46, 47, 145} , {145, 54, 53, 52, 51, 50, 49, 48, 145} , {145, 55, 56, 57, 58, 59, 60, 61, 145} , {145, 68, 67, 66, 65, 64, 63, 62, 145} , {145, 69, 70, 71, 72, 73, 74, 75, 145} , {145, 82, 81, 80, 79, 78, 77, 76, 145} , {145, 83, 84, 85, 86, 87, 88, 89, 145} , {145, 96, 95, 94, 93, 92, 91, 90, 145} , {145, 97, 98, 99, 100, 101, 102, 103, 145} , {145, 110, 109, 108, 107, 106, 105, 104, 145} , {145, 111, 112, 113, 114, 115, 116, 117, 145},  {145, 124, 123, 122, 121, 120, 119, 118, 145} , {145, 125, 126, 127, 128, 129, 130, 131, 145} , {145, 138, 137, 136, 135, 134, 133, 132, 145} , {145, 145, 139, 140, 141, 142, 143, 145, 145} , {145, 145, 145, 145, 144, 145, 145, 145, 145}}; //Sets up the LED display translaton array, first number is the row number, second the column number; values are the corresponding LED light for each of tehse positions
const byte Sprite[3][3] = {{125, 255, 125}, {255, 0, 255}, {125, 255, 125}}; //This 2D array stores the brightness values for a Sprite
bool xposdirection = true; //is the sprite moving to the right (increment x)?
bool yposdirection = true; //is the sprite moving downwards (increment y)?

//For the background
float backgroundcounter;
#define stripethickness 8
float gradient = 0;
bool upwards = true;
float timeval = 0.0000001;
static uint8_t theta = 0;


void setup() {
  Serial.begin(9600);
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS); //This tells the library that there is a strand of WS2812's on DATA_PIN
  FastLED.setBrightness(BRIGHTNESS);
  FastLED.clear();
  delay(500); //safety delay
}

void loop() {

  unsigned long currentmillis = millis(); // Time since activation

  if ((currentmillis - previousbackgroundmillis) >= backgroundframerate) { //check if one background update time has passed
    backgroundcounter = backgroundcounter + 0.1;
    previousbackgroundmillis = currentmillis;
    theta = theta + 10;
    Serial.println(theta);
  }

  if ((currentmillis - previousdisplaymillis) >= framerate) {
    FastLED.show();
    timeval = currentmillis;
    LEDposx = 11.5 + 6 * (sin(timeval / 1000)); //moves the centre around
    LEDposy = 4.5 + 2 * (cos(timeval / 1000));
    updatebackground();
    previousdisplaymillis = currentmillis;
  }

}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Background update routine
void updatebackground() {   // routine was found in https://stackoverflow.com/questions/3365171/calculating-the-angle-between-two-lines-without-having-to-calculate-the-slope
  for (byte i = 0; i <= 22; i++) {
    for (byte j = 0; j <= LEDposy; j++) {
      int r = (LEDarray [i][j]);
      int dx1 = i - LEDposx;
      int dy1 = j - LEDposy;
      float l2 = (dx1 * dx1 + dy1 * dy1);
      float angle2 = acos (dx1 / sqrt(l2)); //gives the angle of pixel (i,j) with the horizontal (1,0) in radians
      int angle3 = angle2 * 40.585; // scale the radian value to integer 0-255 (where 255 corresponds to 2 radians = 3.1415)
      int delta = theta - angle3; //what is the difference of the pixel's angle and the theta value
      if (delta >= 0 && delta < 127) {
        leds[r] = CRGB(0, 0, 255);
      }
      else {
        leds[r] = CRGB(255, 100, 0);
      }
    }
    for (byte j = LEDposy; j <= 8; j++) {
      int r = (LEDarray [i][j]);
      int dx1 = i - LEDposx;
      int dy1 = j - LEDposy;
      float l2 = (dx1 * dx1 + dy1 * dy1);
      float angle2 = acos (dx1 / sqrt(l2)); //gives the angle of pixel (i,j) with the horizontal (1,0) in radians
      int angle3 = angle2 * 40.585; //
      angle3 = 256 - angle3;
      int delta = theta - angle3; //what is the difference of the pixel's angle and the theta value
      leds[r] = CRGB(255, 100, 0);
      if (delta >= 0 && delta < 127) {
        leds[r] = CRGB(0, 0, 255);
      }
      if (delta >= -255 && delta < -127) {
        leds[r] = CRGB(0, 0, 255);
      }
      //      else {
      //        leds[r] = CRGB(255, 100, 0);
      //      }
    }
    //leds[LEDarray[LEDposx][LEDposy]] = CRGB(0, 255, 255); // the centre LED was misbehaving (presumably rounding issues or similar), needs to be manually set on
  }
}
//Background update routine
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
